"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var queue = require("../lib/simqle");
var Rx = require("rxjs");
var Winston = require("winston");
var logger = new Rx.Subject();
var winstonLogger = new Winston.Logger({
    level: 'info',
    transports: [
        new (Winston.transports.Console)()
    ]
});
logger.subscribe(function (l) {
    var awin = winstonLogger;
    awin[l.level].apply(awin[l.level], l.parts);
});
describe('queue', function () {
    it('simple', function (done) {
        // simple
        var q = queue.start(logger, {});
        var calls = 10;
        var pulls = 10;
        var pidx = 0;
        q.q.subscribe(function (data) {
            // console.log('task: idx:pulls', pulls);
            --pulls;
            data.task.subscribe(function (idx) {
                // console.log('task: idx', idx, pidx);
                chai_1.assert.equal(idx, pidx++, '-1-');
            }, function (err) {
                // console.log('task: idx: error', err);
            }, function () {
                // console.log('task: idx: complete');
            });
        });
        (Array(calls).fill(0)).forEach(function (_, idx) {
            // console.log('simple:', idx);
            // const id = Math.random();
            --calls;
            q.push(Rx.Observable.create(function (obs) {
                // console.log('task: vals', id, calls, idx);
                obs.next(idx);
                obs.complete();
            }));
        });
        setTimeout(function () {
            chai_1.assert.equal(0, pulls, 'pulls');
            chai_1.assert.equal(0, calls, 'calls');
            q.stop().subscribe(function () {
                done();
            });
        }, 800);
    });
    it('retries', function (done) {
        // simple 2 retries than ok with measure retryWaittime
        var q = queue.start(logger, {
            reclaimTimeout: 100,
            retryWaitTime: 50,
            maxExecuteCnt: 5,
            taskTimer: 10
        });
        var idxTimer = new Map();
        var calls = 10;
        var retrys = 2;
        (Array(calls).fill(0)).forEach(function (_, idx) {
            // console.log('simple:', idx);
            idxTimer.set(idx, []);
            var cnt = 0;
            q.push(Rx.Observable.create(function (obs) {
                idxTimer.get(idx).push((new Date()).getTime());
                if (cnt++ < retrys) {
                    obs.error(idx);
                }
                else {
                    obs.next(idx);
                    obs.complete();
                }
            }));
        });
        var deadLetters = 0;
        q.deadLetter.subscribe(function () { ++deadLetters; });
        setTimeout(function () {
            chai_1.assert.equal(0, deadLetters, 'calls != deadletter');
            idxTimer.forEach(function (v, k) {
                chai_1.assert.equal(retrys + 1, v.length);
                var diff = v.map(function (x, i, vas) {
                    if (i < 1) {
                        return 0;
                    }
                    return x - vas[i - 1];
                }).slice(1);
                // console.log('diff:', diff, v);
                diff.forEach(function (d) {
                    chai_1.assert.isTrue(q.retryWaitTime <= d && d <= (q.retryWaitTime * 1.5));
                });
            });
            // assert.equal(0, calls);
            q.stop().subscribe(done);
        }, 400);
    });
    it('reclaim', function (done) {
        // simple 2 retries than ok with measure retryWaittime
        var q = queue.start(logger, {
            reclaimTimeout: 100,
            retryWaitTime: 50,
            maxExecuteCnt: 5,
            taskTimer: 10
        });
        var idxTimer = new Map();
        var calls = 10;
        var retrys = 2;
        (Array(calls).fill(0)).forEach(function (_, idx) {
            // console.log('simple:', idx);
            idxTimer.set(idx, []);
            var cnt = 0;
            q.push(Rx.Observable.create(function (obs) {
                idxTimer.get(idx).push((new Date()).getTime());
                if (cnt++ >= retrys) {
                    obs.next(idx);
                }
            }));
        });
        var deadLetters = 0;
        q.deadLetter.subscribe(function () { ++deadLetters; });
        setTimeout(function () {
            chai_1.assert.equal(0, deadLetters, 'calls != deadletter');
            idxTimer.forEach(function (v, k) {
                chai_1.assert.equal(retrys + 1, v.length);
                var diff = v.map(function (x, i, vas) {
                    if (i < 1) {
                        return 0;
                    }
                    return x - vas[i - 1];
                }).slice(1);
                // console.log('diff:', diff, v);
                diff.forEach(function (d) {
                    chai_1.assert.isTrue(q.reclaimTimeout <= d && d <= (q.reclaimTimeout * 1.5));
                });
            });
            // assert.equal(0, calls);
            q.stop().subscribe(done);
        }, 500);
    });
    it('error', function (done) {
        // simple 5 error remove
        var q = queue.start(logger, {
            reclaimTimeout: 100,
            retryWaitTime: 50,
            maxExecuteCnt: 5,
            taskTimer: 10
        });
        var idxTimer = new Map();
        var calls = 10;
        (Array(calls).fill(0)).forEach(function (_, idx) {
            // console.log('simple:', idx);
            idxTimer.set(idx, []);
            q.push(Rx.Observable.create(function (obs) {
                // --calls;
                idxTimer.get(idx).push((new Date()).getTime());
                obs.error(idx);
                // obs.complete();
            }));
        });
        var deadLetters = 0;
        q.deadLetter.subscribe(function () { ++deadLetters; });
        setTimeout(function () {
            chai_1.assert.equal(calls, deadLetters, 'calls != deadletter');
            idxTimer.forEach(function (v, k) {
                chai_1.assert.equal(q.maxExecuteCnt, v.length);
                var diff = v.map(function (x, i, vas) {
                    if (i < 1) {
                        return 0;
                    }
                    return x - vas[i - 1];
                }).slice(1);
                // console.log('diff:', diff, v);
                diff.forEach(function (d) {
                    chai_1.assert.isTrue(q.retryWaitTime <= d && d <= (q.retryWaitTime * 1.5));
                });
            });
            // assert.equal(0, calls);
            q.stop().subscribe(done);
        }, 800);
    });
});
//# sourceMappingURL=simqle-test.js.map